/*
 * this is a simple test app for playing around with the xvideo extention
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "config.h"
#include "grab.h"
#include "xv.h"

#ifndef HAVE_LIBXV
/* dummy stubs */
int have_xv;
void xv_init()
{
    if (debug)
	fprintf(stderr,"Xv: compiled without Xvideo extention support\n");
}
void xv_video(Window win, int width, int height, int on) {}
#else

/* ********************************************************************* */
/* the real code                                                         */

#include <X11/extensions/Xv.h>
#include <X11/extensions/Xvlib.h>

extern Display    *dpy;
int               have_xv;

static int              ver, rel, req, ev, err;
static int              adaptors;
static int              encodings;
static int              adaptor = -1, port = -1;
static XvAdaptorInfo    *ai;
static XvEncodingInfo   *ei;

static int  xv_error;
static Atom xv_encoding;
static Atom xv_hue;
static Atom xv_saturation;
static Atom xv_brightness;
static Atom xv_contrast;
static Atom xv_freq;
static Atom xv_mute;
static Atom xv_volume;
static Atom xv_foobar;

static struct STRTAB *norms;
static struct STRTAB *inputs;
static int my_norm=-1, my_input=-1;

static struct ENC_MAP {
    int norm;
    int input;
    int encoding;
} *enc_map;

static struct GRABBER xv = {
    "Xvideo",
    0,
};

static int
xv_overlay(int x, int y, int width, int height, int format,
	   struct OVERLAY_CLIP *oc, int count)
{
    if (debug)
	fprintf(stderr,"Ouch: xv_overlay called\n");
    return 0;
}

static int
xv_error_dev_null(Display * dpy, XErrorEvent * event)
{
    xv_error++;
    return 0;
}

static int
xv_probe(XvPortID port, Atom attr)
{
    void            *old_handler;
    int              value;

    xv_error = 0;
    old_handler = XSetErrorHandler(xv_error_dev_null);
    XvGetPortAttribute(dpy, port, attr, &value);
    XSync(dpy, False);
    XSetErrorHandler(old_handler);
    if (debug)
	fprintf(stderr,"xv: probe: %s: %s\n",
		XGetAtomName(dpy,attr), xv_error ? "no" : "yes");
    return (xv_error > 0) ? 0 : 1;
}

/* map: 0 - 65536  =>  -1000 - 1000 */
static int me_to_xv(int val) {
    val = val * 2000 / 65536 - 1000;
    if (val < -1000) val = -1000;
    if (val >  1000) val =  1000;
    return val;
}

static int
xv_input(int input, int norm)
{
    int i;
    
    if (-1 != norm)
	my_norm  = norm;
    if (-1 != input)
	my_input = input;
    for (i = 0; i < encodings; i++) {
	if (enc_map[i].norm  == my_norm &&
	    enc_map[i].input == my_input) {
	    XvSetPortAttribute(dpy,port,xv_encoding,enc_map[i].encoding);
	    return 0;
	}
    }
    return 0;
}

static int
xv_picture(int color, int bright, int hue, int contrast)
{
    if (-1 != bright && xv_brightness != None)
	XvSetPortAttribute(dpy,port,xv_brightness,me_to_xv(bright));
    if (-1 != hue && xv_hue != None)
	XvSetPortAttribute(dpy,port,xv_hue,me_to_xv(hue));
    if (-1 != contrast && xv_contrast != None)
	XvSetPortAttribute(dpy,port,xv_contrast,me_to_xv(contrast));
    return 0;
}

static int
xv_tune(unsigned long freq)
{
    XvSetPortAttribute(dpy,port,xv_freq,freq);
    return 0;
}

int
xv_audio(int mute, int volume, int *mode)
{
    if (-1 == mute && -1 == volume && mode && -1 == *mode) {
	/* only read mode */
	*mode = 0;
	return 0;
    }
    if (-1 != mute && xv_mute != None) {
	XvSetPortAttribute(dpy,port,xv_mute,mute);
    }
    if (-1 != volume && xv_volume != None) {
	XvSetPortAttribute(dpy,port,xv_volume,me_to_xv(volume));
    }
    return 0;
}

int
xv_close()
{
    int mode = -1;
    if (debug)
	fprintf(stderr,"Xv: close\n");
    xv_audio(1, -1, &mode);
    return 0;
}

void
xv_video(Window win, int width, int height, int on)
{
    static GC gc;
    
    if (debug)
	fprintf(stderr,"Xv: video: win=0x%lx, size=%dx%d, %s\n",
		win, width, height, on ? "on" : "off");
    if (on) {
	if (!gc)
	    gc = XCreateGC(dpy, win, 0, NULL);
	XvPutVideo(dpy,port,win,gc,0,0,width,height,0,0,width,height);
    } else {
	XvStopVideo(dpy,port,win);
    }
}

static int
xv_strlist_add(struct STRTAB **tab, char *str)
{
    int i;

    if (NULL == *tab) {
	*tab = malloc(sizeof(struct STRTAB)*2);
	i = 0;
    } else {
	for (i = 0; (*tab)[i].str != NULL; i++)
	    if (0 == strcasecmp((*tab)[i].str,str))
		return (*tab)[i].nr;
	*tab = realloc(*tab,sizeof(struct STRTAB)*(i+2));
    }
    (*tab)[i].nr  = i;
    (*tab)[i].str = strdup(str);
    (*tab)[i+1].nr  = -1;
    (*tab)[i+1].str = NULL;
    return i;
}

void xv_init()
{
    char norm[32],input[32];
    int i;
    
    if (Success != XvQueryExtension(dpy,&ver,&rel,&req,&ev,&err)) {
	if (debug)
	    fprintf(stderr,"Xv: Server has no Xvideo extention support\n");
	return;
    }
    if (Success != XvQueryAdaptors(dpy,DefaultRootWindow(dpy),&adaptors,&ai)) {
	fprintf(stderr,"Xv: XvQueryAdaptors failed");
	exit(1);
    }
    if (debug)
	fprintf(stderr,"Xv: %d adaptors available.\n",adaptors);
    for (i = 0; i < adaptors; i++) {
	if (debug)
	    fprintf(stderr,"Xv: %s:%s%s, ports %ld-%ld\n",
		    /* ai[i].name */ "fixme",
		    (ai[i].type & XvInputMask)  ? " input"  : "",
		    (ai[i].type & XvOutputMask) ? " output" : "",
		    ai[i].base_id,
		    ai[i].base_id+ai[i].num_ports-1);
	if ((ai[i].type & XvInputMask) && (port == -1)) {
	    port = ai[i].base_id;
	    adaptor = i;	    
	}
    }
    if (port == -1) {
	if (debug)
	    fprintf(stderr,"Xv: no usable port found\n");
	return;
    }

    /* probe was successfull */
    fprintf(stderr,"Xv: using port %d\n",port);
    have_xv = 1;
    xv.grab_overlay = xv_overlay;
    grabber = &xv;

    /* query encoding list */
    if (Success != XvQueryEncodings(dpy, port, &encodings, &ei)) {
	fprintf(stderr,"Oops: XvQueryEncodings failed\n");
	exit(1);
    }
    enc_map = malloc(sizeof(*enc_map)*encodings);
    for (i = 0; i < encodings; i++) {
	if (2 == sscanf(ei[i].name,"%31[^-]-%31s",norm,input)) {
	    enc_map[i].norm     = xv_strlist_add(&norms,norm);
	    enc_map[i].input    = xv_strlist_add(&inputs,input);
	    enc_map[i].encoding = ei[i].encoding_id;
	} else {
	    /* FIXME */
	}
    }
    xv.norms = norms;
    xv.inputs = inputs;
    
    /* build atoms */
    xv_encoding   = XInternAtom(dpy, "XV_ENCODING", False);
    xv_hue        = XInternAtom(dpy, "XV_HUE", False);
    xv_saturation = XInternAtom(dpy, "XV_SATURATION", False);
    xv_brightness = XInternAtom(dpy, "XV_BRIGHTNESS", False);
    xv_contrast   = XInternAtom(dpy, "XV_CONTRAST", False);
    xv_freq       = XInternAtom(dpy, "XV_FREQ", False);
    xv_mute       = XInternAtom(dpy, "XV_MUTE", False);
    xv_volume     = XInternAtom(dpy, "XV_VOLUME", False);
    xv_foobar     = XInternAtom(dpy, "XV_FOOBAR", False);

    /* check attributes */
    xv_probe(port,xv_encoding)   || (xv_encoding   = None);
    xv_probe(port,xv_hue)        || (xv_hue        = None);
    xv_probe(port,xv_saturation) || (xv_saturation = None);
    xv_probe(port,xv_brightness) || (xv_brightness = None);
    xv_probe(port,xv_contrast)   || (xv_contrast   = None);
    xv_probe(port,xv_freq)       || (xv_freq       = None);
    xv_probe(port,xv_mute)       || (xv_mute       = None);
    xv_probe(port,xv_volume)     || (xv_volume     = None);
    xv_probe(port,xv_foobar)     || (xv_foobar     = None);

    /* set hooks */
    if (xv_encoding != None) {
	xv.grab_input = xv_input;
    }
    if (xv_hue        != None ||
	xv_brightness != None ||
	xv_contrast   != None) {
	xv.grab_picture = xv_picture;
    }
    if (xv_freq != None) {
	xv.grab_tune = xv_tune;
    }
    if (xv_mute   != None ||
	xv_volume != None ) {
	xv.grab_audio = xv_audio;
	xv.grab_close = xv_close;
    }
}

#endif
